﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (string.IsNullOrEmpty(Params.DavUrlPrefix) || !Params.AddRoutingConfig)  return String.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;

namespace ITHit.WebDav.Server.SampleModel.Config
{
    /// <summary>
    /// Configuration section which specifies which paths/verbs shall be disabled
    /// in ASP.NET MVC routing and handled by <see cref="DavHandler"/> handler.
    /// </summary>
    public class DavLocationsSection : ConfigurationSection
    {
        private static ConfigurationPropertyCollection properties = new ConfigurationPropertyCollection();
        private static readonly ConfigurationProperty propLocations = new ConfigurationProperty(null, typeof(DavLocationCollection), null, ConfigurationPropertyOptions.IsDefaultCollection);

        static DavLocationsSection()
        {
            properties.Add(propLocations);
        }

        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return properties;
            }
        }

        /// <summary>
        /// Gets collection of WebDAV locations.
        /// </summary>
        [ConfigurationProperty("", IsDefaultCollection = true)]
        public DavLocationCollection Locations
        {
            get
            {
                return (DavLocationCollection)base[propLocations];
            }
        }

        /// <summary>
        /// Gets this configuration section from config file.
        /// Returns null if the section is absent.
        /// </summary>
        public static DavLocationsSection Instance
        {
            get
            {
                return (DavLocationsSection)ConfigurationManager.GetSection("davLocations");
            }
        }
    }
}
