//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
//!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>
using System.Security.Principal;
//!<#}#>
using System.Web;
using System.Xml;
using System.Xml.Serialization;

using ITHit.WebDAV.Server;
//!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl;
//!<#}#>

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IHttpHandler
    /// <summary>
    /// This handler processes WebDAV requests.
    /// </summary>
    public class DavHandler : IHttpHandler
    {
        /// <summary>
        /// If debug logging is enabled reponses are output as formatted XML,
        /// all requests and response headers and most bodies are logged.
        /// If debug logging is disabled only errors are logged.
        /// </summary>
        private static readonly bool debugLoggingEnabled =
            "true".Equals(
                ConfigurationManager.AppSettings["DebugLoggingEnabled"],
                StringComparison.InvariantCultureIgnoreCase);

        //!<# if (Params.Storage == StorageType.Ntfs) { #>
        /// <summary>
        /// Path to the folder which stores WebDAV files.
        /// </summary>
        private static readonly string repositoryPath =
            (ConfigurationManager.AppSettings["RepositoryPath"] ?? string.Empty).TrimEnd('\\');
        //!<#}#>

        //!<# if (false && Params.ACL && Params.Storage == StorageType.Ntfs) { #>
        /// <summary>
        /// Domain(or computer name) agains which to perform authentication.
        /// </summary>
        private static readonly string domain = ConfigurationManager.AppSettings["Domain"];

        /// <summary>
        /// Name of user to be used with anonymous access to webdav.
        /// </summary>
        private static readonly string anonymousUserLogin = ConfigurationManager.AppSettings["AnonymousUserLogin"];

        /// <summary>
        /// Password for <see cref="anonymousUserLogin"/>.
        /// </summary>
        private static readonly string anonymousUserPassword = 
            ConfigurationManager.AppSettings["AnonymousUserPassword"];

        /// <summary>
        /// <see cref="WindowsIdentity"/> which corresponds to the anonymous user.
        /// </summary>
        private WindowsIdentity anonymousUser;
        //!<#}#>

        //!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>
        /// <summary>
        /// Initializes serialization assemblies.
        /// </summary>
        static DavHandler()
        {
            //create serialization assemblies here.
            //Otherwise it fails with permissions error on first access in impersonated context.
            using (MemoryStream ms = new MemoryStream())
            {
                new XmlSerializer(typeof(List<DateLockInfo>)).Serialize(ms, new List<DateLockInfo>());
                new XmlSerializer(typeof(List<PropertyValue>)).Serialize(ms, new List<PropertyValue>());
                new XmlSerializer(typeof(long)).Serialize(ms, 0L);
                new XmlSerializer(typeof(int)).Serialize(ms, 0);
            }
        }
        //!<# } #> 

        /// <summary>
        /// Gets a value indicating whether another request can use the
        /// <see cref="T:System.Web.IHttpHandler"/> instance.
        /// </summary>
        /// <returns>
        /// true if the <see cref="T:System.Web.IHttpHandler"/> instance is reusable; otherwise, false.
        /// </returns>
        public bool IsReusable
        {
            get { return true; }
        }

        /// <summary>
        /// Enables processing of HTTP Web requests.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpContext"/> object that provides references to the
        /// intrinsic server objects (for example, Request, Response, Session, and Server) used to service
        /// HTTP requests. 
        /// </param>
        public void ProcessRequest(HttpContext context)
        {
            DavEngine engine = getOrInitializeEngine(context);

            context.Response.BufferOutput = false;

            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            string physicalRepositoryPath = repositoryPath.StartsWith("~") ?
                context.Server.MapPath(repositoryPath) : repositoryPath;
            var ntfsDavContext = new __NtfsStorage__.DavContext(context, physicalRepositoryPath, engine.Logger);
            //!<#}#>
            //!<# if (false && Params.Storage == StorageType.Ntfs && Params.ACL) { #>
            ntfsDavContext.AnonymousUser = anonymousUser;
            engine.Logger.LogDebug(
                string.Format(
                "Logged in as:{0}, anonymous user is:{1}",
                ntfsDavContext.User.Name, anonymousUser == null ? "null" : anonymousUser.Name));
            //!<#}#>
            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            engine.Run(ntfsDavContext);
            //!<#}#>

            //!<# if (Params.Storage == StorageType.Db) { #>
            using (var sqlDavContext = new __SqlStorage__.DavContext(context))
            {
                engine.Run(sqlDavContext);
            }
            //!<#}#>

            //!<# if (Params.Storage == StorageType.Blank) { #>
            var davContext = new __DavImpl__.DavContext(context, engine.Logger);
            engine.Run(davContext);
            //!<#}#>
        }

        /// <summary>
        /// Initializes engine.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpContext"/>.</param>
        /// <returns>Initialized <see cref="DavEngine"/>.</returns>
        private DavEngine initializeEngine(HttpContext context)
        {
            //!<# if (false && Params.ACL && Params.Storage == StorageType.Ntfs) { #>
            try
            {
                anonymousUser = LogonUtil.GetUser(anonymousUserLogin, domain, anonymousUserPassword);
            }
            catch (Exception ex)
            {
                Logger.Instance.LogError("Failed to retrieve anonymous user identity", ex);
                throw;
            }
            //!<#}#>

            var engine = new DavEngine
            {
                Logger = Logger.Instance,
                //Use idented responses if debug logging is enabled.
                OutputXmlFormatting = debugLoggingEnabled ? Formatting.Indented : Formatting.None,
            };

            //!<# if (!string.IsNullOrEmpty(Params.License)) { #>
            //!            engine.License = @"<#=Params.License == null ? null : Params.License.Replace("\"", "\"\"")#>";
            //!<# } else { #>
            engine.License = File.ReadAllText(context.Request.PhysicalApplicationPath + "License.lic");
            //!<# } #>

            //!<# if (Params.AddCustomGetHandler) { #>
            // set custom handler to process GET requests to folders and display info about how to connect to server
            var handler = new MyCustomGetHandler();
            handler.OriginalHandler = engine.RegisterMethodHandler("GET", handler);
            //!<# } #>
            return engine;
        }

        /// <summary>
        /// Initializes or gets engine singleton.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpContext"/>.</param>
        /// <returns>Instance of <see cref="DavEngine"/>.</returns>
        private DavEngine getOrInitializeEngine(HttpContext context)
        {
            //we don't use any double check lock pattern here because nothing wrong
            //is going to happen if we created occasionally several engines.
            const string ENGINE_KEY = "$DavEngine$";
            if (context.Application[ENGINE_KEY] == null)
            {
                context.Application[ENGINE_KEY] = initializeEngine(context);
            }

            return (DavEngine)context.Application[ENGINE_KEY];
        }
    }
    //$>
}
