﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;
using ITHit.WebDAV.Server.Acl;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav
{   
    public class CalFolder : DavFolder, ICalendarFolder
    {
        private readonly DirectoryInfo dirInfo;
        //private readonly DavContext context;
        private readonly string userName;

        private static string getLocalPath(DavContext context, string userName, string name)
        {
            var calendars = System.IO.Path.Combine(context.RepositoryPath, "calendars");
            var userFolder = System.IO.Path.Combine(calendars, userName);
            var path = System.IO.Path.Combine(userFolder, name);
            return path;
        }

        private static string getPath(string userName, string name)
        {
            return "calendars/users/" + EncodeUtil.EncodeUrlPart(userName) + "/" + EncodeUtil.EncodeUrlPart(name) + "/";
        }

        public CalFolder(DavContext context, string userName, string name):
            this(new DirectoryInfo(getLocalPath(context, userName, name)), context, getPath(userName, name), userName)

        {
            this.userName = userName;            
        }

        private CalFolder(DirectoryInfo dirInfo, DavContext context, string path, string userName) :base(dirInfo, context, path)
        {
            this.userName = userName;
            this.dirInfo = dirInfo;
        }

        public bool Exists
        {
            get { return dirInfo.Exists; }
        }

        #region ICalendarFolder Members

        public IHierarchyItem CreateCalendarResource(string name)
        {
            string fileName = System.IO.Path.Combine(dirInfo.FullName, name);

            context.FileOperation(this,
                () => new FileStream(fileName, FileMode.CreateNew).Dispose(),
                Privilege.Bind);

            return new CalendarResource(new FileInfo(fileName), context, Path + EncodeUtil.EncodeUrlPart(name));
        }

        public string CalendarDescription
        {
            get
            {
                return "Some Description";
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public string CalendarTimeZone
        {
            get
            {
                return @"BEGIN:VCALENDAR
PRODID:-//Example Corp.//CalDAV Client//EN
VERSION:2.0
BEGIN:VTIMEZONE
TZID:US-Eastern
LAST-MODIFIED:19870101T000000Z
BEGIN:STANDARD
DTSTART:19671029T020000
RRULE:FREQ=YEARLY;BYDAY=-1SU;BYMONTH=10
TZOFFSETFROM:-0400
TZOFFSETTO:-0500
TZNAME:Eastern Standard Time (US &amp; Canada)
END:STANDARD
BEGIN:DAYLIGHT
DTSTART:19870405T020000
RRULE:FREQ=YEARLY;BYDAY=1SU;BYMONTH=4
TZOFFSETFROM:-0500
TZOFFSETTO:-0400
TZNAME:Eastern Daylight Time (US &amp; Canada)
END:DAYLIGHT
END:VTIMEZONE
END:VCALENDAR";
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public IEnumerable<string> SupportedComponents
        {
            get
            {
                yield return "VEVENT";
                yield return "VTODO";
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public IEnumerable<CalendarData> SupportedCalendarData
        {
            get
            {
                yield return new CalendarData("text/calendar", "2.0");
            }
        }

        public long MaxResourceSize
        {
            get { return int.MaxValue; }
        }

        public DateTime MinDateTime
        {
            get { return DateTime.MinValue; }
        }

        public DateTime MaxDateTime
        {
            get { return DateTime.MaxValue; }
        }

        public long MaxInstances
        {
            get { return long.MaxValue; }
        }

        public long MaxAttendeesPerInstance
        {
            get { return long.MaxValue; }
        }

        #endregion

        #region IItemCollection Members

        public override IEnumerable<IHierarchyItem> GetChildren(IList<PropertyName> propNames)
        {
            FileSystemInfo[] fileInfos =
                context.FileOperation(this,
                    () => this.dirInfo.GetFileSystemInfos(),
                    Privilege.Read);


            foreach (FileSystemInfo item in fileInfos)
            {
                var childPath = Path + EncodeUtil.EncodeUrlPart(item.Name);                
                if (item is FileInfo)
                {
                    yield return new DavFile((FileInfo)item, context, childPath);
                }
            }            
        }

        #endregion
    }
}